<?php
error_reporting(E_ALL);
ini_set('display_errors', TRUE);

function abort($body, $status = NULL) {
	if ($status) header("HTTP/1.1 $status");
	print "$body\n";
	exit;
}

function getoutput($blocks = [])
{
	$doc = ob_get_clean();

	if (!empty($blocks['warn'])) {
		$warn = '<p class="warn">[[warn]]</p>';
		if ($offset = strpos($doc, '</h2>')) {
			$doc = substr_replace($doc, "\n\n".$warn, $offset + 5, 0);
		}
		else {
			$doc = $warn . "\n\n" . $doc;
		}
	}

	return preg_replace_callback(
		'< \[\[ ([^] ]+) ([^]]*) \]\] >x',
		function ($sub) use ($blocks) {
			list ($placeholder, $name, $params) = $sub;
			if (isset($blocks[$name])) {
				$html = $blocks[$name];
			}
			elseif (file_exists("$name.php")) {
				ob_start();
				$Page = $GLOBALS['Page'] . $GLOBALS['Args'];
				$Args = '';
				$Place = $GLOBALS['Place'];
				foreach (explode(' ', $params) as $param) {
					if ($set = strpos($param, '=')) {
						$Place[ substr($param, 0, $set) ] = substr($param, $set + 1);
					}
					elseif (!empty($param)) {
						$Args .= '/'.$param;
					}
				}
				try {
					include "$name.php";
					$html = ob_get_clean();
				}
				catch (Exception $e) {
					$html = sprintf('<strong class="warn">%s</strong>',
						"fout in <em>$name</em>: {$e->getMessage()}"
					);
				}
			}
			else {
				$html = '<strong class="warn"><em>'.$name.'</em> ontbreekt</strong>';
			}
			return sprintf('<!--BLOCK:%s-->%s<!--/-->',
				is_numeric($name) ? '' : $placeholder, # edit replacement
				preg_replace('{<!--[^-]*-->}', '', $html) # contents
			);
		},
		$doc
	);
}

# custom error handling

define('DOCROOT', getcwd());
set_include_path(implode(PATH_SEPARATOR, [ DOCROOT, __DIR__ ]));

function fail($error)
{
	http_response_code(500);
	include_once 'page.inc.php';
	ob_start();
	require_once '500.html';
	print getoutput(['debug' => $error]);
}

set_exception_handler('fail');

define('E_FATAL', E_ERROR | E_CORE_ERROR | E_COMPILE_ERROR | E_USER_ERROR);

set_error_handler(function ($level, $error, $file, $line) {
	if ($level & E_FATAL) {
		fail($error);
		return;
	}
	return FALSE;
});

register_shutdown_function(function () {
	# display failure page for fatal exceptions
	$error = error_get_last();
	if (!($error['type'] & E_FATAL)) return;
	fail("Fatal: $error[message] in $error[file]:$error[line]");
});

error_reporting(error_reporting() & ~E_FATAL);

# user login and control

include_once 'auth.inc.php';
$Edit = isset($_GET['edit']);

# distinguish subpage Args from topmost Page script

$Args = '';
$Page = preg_replace('/\?.*/', '', @$_SERVER['PATH_INFO'] ?: $_SERVER['REQUEST_URI']);
$Page = urldecode(trim($Page, '/')) ?: 'index';
while (TRUE) {
	if (file_exists("$Page/.private")) {
		# access restriction
		if (empty($User)) {
			http_response_code(303);
			$target = urlencode($_SERVER['REQUEST_URI']);
			header("Location: /login?goto=$target");
			exit;
		}
		$PageAccess = $Page;
	}

	if (file_exists("$Page/index.php")) {
		break;
	}

	$up = strrpos($Page, '/');
	$Args = substr($Page, $up) . $Args;
	$Page = substr($Page, 0, $up);
	if ($up === FALSE) {
		break;
	}
}

# load static contents

ob_start(); # page body
ob_start(); # inner html
print '<div class="static">'."\n\n";

$found = FALSE;
if (file_exists("$Page$Args.html")) {
	$found = include "./$Page$Args.html";
}
elseif (file_exists("$Page$Args/index.html")) {
	$found = include "./$Page$Args/index.html";
}
elseif (!empty($User['admin'])) {
	$found = include (file_exists("$Page/template.html") ? "$Page/template.html" : './template.html');
}

print "</div>\n\n";

# execute dynamic code

$Place = [];

if ($Page) {
	$found |= require "./$Page/index.php";
}

$Place += [
	'user'  => empty($User) ? '' : $User['name'],
	'url'   => htmlspecialchars($_SERVER['REQUEST_URI']),
];

# global html

if (!$found) {
	# no resulting output
	http_response_code(404);
	@require '404.html';
}

include_once 'page.inc.php';

